/**
 * Copyright (c) 2012, Patrick Cornelissen (orchit e.K.)
 * All rights reserved.
 *
 * Redistribution and use in source and binary forms, with or without
 * modification, are permitted provided that the following conditions are met:
 *
 * 1. Redistributions of source code must retain the above copyright notice, this
 * list of conditions and the following disclaimer.
 * 2. Redistributions in binary form must reproduce the above copyright notice,
 * this list of conditions and the following disclaimer in the documentation
 * and/or other materials provided with the distribution.
 *
 * THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS "AS IS" AND
 * ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE IMPLIED
 * WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE ARE
 * DISCLAIMED. IN NO EVENT SHALL THE COPYRIGHT OWNER OR CONTRIBUTORS BE LIABLE FOR
 * ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES
 * (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES;
 * LOSS OF USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND
 * ON ANY THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT
 * (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE OF THIS
 * SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
 *
 * The views and conclusions contained in the software and documentation are those
 * of the authors and should not be interpreted as representing official policies,
 * either expressed or implied, of the FreeBSD Project.
 */
package de.orchit.securitytools.password;


public enum PasswordQuality {
	insufficient(0), minimal(10), medium(20), good(28), perfect(40);
	private final int minScore;

	PasswordQuality(int minScore) {
		this.minScore = minScore;
	}

	private static PasswordQuality byScore(int score) {
		if (score >= perfect.minScore) return perfect;
		if (score >= good.minScore) return good;
		if (score >= medium.minScore) return medium;
		if (score >= minimal.minScore) return minimal;
		return insufficient;
	}

	public static PasswordQuality determineQuality(String password) {
		int score = 0;
		score += calcLengthScore(password);
		score += calcSpecialCharScore(password);
		score += calcLowercaseCharScore(password);
		score += calcUppercaseCharScore(password);
		score += calcNumberScore(password);
		return PasswordQuality.byScore(score);
	}

	static int calcNumberScore(String password) {
		String remainder = password.replaceAll("[^0-9]", "");
		if (remainder.length() == 0) return -10;
		return Math.min(5, remainder.length() * 2);
	}

	static int calcUppercaseCharScore(String password) {
		String remainder = password.replaceAll("[^A-Z]", "");
		if (remainder.length() == 0) return -10;
		return Math.min(5, remainder.length() * 2);
	}

	static int calcLowercaseCharScore(String password) {
		String remainder = password.replaceAll("[^a-z]", "");
		if (remainder.length() == 0) return -10;
		return Math.min(5, remainder.length() * 2);
	}

	static int calcSpecialCharScore(String password) {
		String remainder = password.replaceAll("[a-zA-Z0-9]", "");
		if (remainder.length() == 0) return -10;
		return Math.min(10, remainder.length() * 2);
	}

	static int calcLengthScore(String password) {
		int length = password.length();
		if (length < 6) return -100;
		if (length == 6) return 0;
		if (length >= 30) return 30;
		return (int) Math.round(length - Math.sqrt(length));
	}

}
