/**
 * Copyright (c) 2012, Patrick Cornelissen (orchit e.K.)
 * All rights reserved.
 *
 * Redistribution and use in source and binary forms, with or without
 * modification, are permitted provided that the following conditions are met:
 *
 * 1. Redistributions of source code must retain the above copyright notice, this
 * list of conditions and the following disclaimer.
 * 2. Redistributions in binary form must reproduce the above copyright notice,
 * this list of conditions and the following disclaimer in the documentation
 * and/or other materials provided with the distribution.
 *
 * THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS "AS IS" AND
 * ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE IMPLIED
 * WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE ARE
 * DISCLAIMED. IN NO EVENT SHALL THE COPYRIGHT OWNER OR CONTRIBUTORS BE LIABLE FOR
 * ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES
 * (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES;
 * LOSS OF USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND
 * ON ANY THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT
 * (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE OF THIS
 * SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
 *
 * The views and conclusions contained in the software and documentation are those
 * of the authors and should not be interpreted as representing official policies,
 * either expressed or implied, of the FreeBSD Project.
 */
package de.orchit.securitytools.password.impl;

import de.orchit.securitytools.password.PasswordHashService;
import de.orchit.securitytools.password.PasswordServiceException;
import org.apache.commons.codec.binary.Base64;

import java.io.IOException;
import java.io.UnsupportedEncodingException;
import java.security.MessageDigest;
import java.security.NoSuchAlgorithmException;
import java.security.SecureRandom;
import java.util.Arrays;

public class OwaspPasswordHashService implements PasswordHashService {
	private final static int ITERATIONS = 1500;

	@Override
	public boolean checkPassword(String password, String hash) throws PasswordServiceException {
		try {
			String salt = hash.substring(0, hash.indexOf('@'));
			String iterationsAndHash = hash.substring(hash.indexOf('@') + 1);
			int iterations = 100*Integer.valueOf(iterationsAndHash.substring(0, iterationsAndHash.indexOf('#')));
			String digest = iterationsAndHash.substring(iterationsAndHash.indexOf('#') + 1);
			if (digest == null || salt == null || iterations < 1000) {
				throw new RuntimeException("Database inconsistent salt or digested password altered");
			}
			byte[] bDigest = base64ToByte(digest);
			byte[] bSalt = base64ToByte(salt);

			// Compute the new DIGEST
			byte[] proposedDigest;

			proposedDigest = getHash(iterations, password, bSalt);
			return Arrays.equals(proposedDigest, bDigest);
		} catch (NoSuchAlgorithmException e) {
			throw new PasswordServiceException(e);
		} catch (UnsupportedEncodingException e) {
			throw new PasswordServiceException(e);
		} catch (IOException e) {
			throw new PasswordServiceException(e);
		}
	}


	@Override
	public String calculateHash(String password) throws PasswordServiceException {
		return calculateHash(password, ITERATIONS);

	}


	public String calculateHash(String password, int iterations) throws PasswordServiceException {
		try {
			// Uses a secure Random not a simple Random
			SecureRandom random =  SecureRandom.getInstance("SHA1PRNG");
			// Salt generation 64 bits long
			byte[] bSalt = new byte[8];
			random.nextBytes(bSalt);
			// Digest computation
			byte[] bDigest = getHash(iterations, password, bSalt);
			String sDigest = byteToBase64(bDigest);
			String sSalt = byteToBase64(bSalt);
			return sSalt + '@' + iterations/100 + '#' + sDigest;
		} catch (NoSuchAlgorithmException e) {
			throw new PasswordServiceException(e);
		} catch (UnsupportedEncodingException e) {
			throw new PasswordServiceException(e);
		}
	}


	/**
	 * From a password, a number of iterations and a salt,
	 * returns the corresponding digest
	 *
	 * @param iterationNb int The number of iterations of the algorithm
	 * @param password    String The password to encrypt
	 * @param salt        byte[] The salt
	 * @return byte[] The digested password
	 * @throws NoSuchAlgorithmException If the algorithm doesn't exist
	 */
	private byte[] getHash(int iterationNb, String password, byte[] salt) throws NoSuchAlgorithmException, UnsupportedEncodingException {
		MessageDigest digest = MessageDigest.getInstance("SHA-256");
		digest.reset();
		digest.update(salt);
		byte[] input = digest.digest(password.getBytes("UTF-8"));
		for (int i = 0; i < iterationNb; i++) {
			digest.reset();
			input = digest.digest(input);
		}
		return input;
	}


	/**
	 * From a base 64 representation, returns the corresponding byte[]
	 *
	 * @param data String The base64 representation
	 * @return byte[]
	 * @throws IOException
	 */
	private static byte[] base64ToByte(String data) throws IOException {
		return Base64.decodeBase64(data);
	}

	/**
	 * From a byte[] returns a base 64 representation
	 *
	 * @param data byte[]
	 * @return String
	 * @throws IOException
	 */
	private static String byteToBase64(byte[] data) {
		return Base64.encodeBase64String(data);
	}


}
