/**
 * Copyright (c) 2012, Patrick Cornelissen (orchit e.K.)
 * All rights reserved.
 *
 * Redistribution and use in source and binary forms, with or without
 * modification, are permitted provided that the following conditions are met:
 *
 * 1. Redistributions of source code must retain the above copyright notice, this
 * list of conditions and the following disclaimer.
 * 2. Redistributions in binary form must reproduce the above copyright notice,
 * this list of conditions and the following disclaimer in the documentation
 * and/or other materials provided with the distribution.
 *
 * THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS "AS IS" AND
 * ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE IMPLIED
 * WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE ARE
 * DISCLAIMED. IN NO EVENT SHALL THE COPYRIGHT OWNER OR CONTRIBUTORS BE LIABLE FOR
 * ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES
 * (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES;
 * LOSS OF USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND
 * ON ANY THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT
 * (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE OF THIS
 * SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
 *
 * The views and conclusions contained in the software and documentation are those
 * of the authors and should not be interpreted as representing official policies,
 * either expressed or implied, of the FreeBSD Project.
 */
package de.orchit.securitytools.password;

import org.junit.Test;

import java.util.Arrays;
import java.util.List;

import static org.hamcrest.CoreMatchers.is;
import static org.junit.Assert.assertThat;


public class PasswordQualityTest {
	@Test
	public void testDetermineQualityInsufficient() throws Exception {
		List<String> badPwds = Arrays.asList(
				"äö=78DF",//too easy
				"1234567890",//too easy
				"abcdefghijklmnopqrstuvwxyz",//too easy
				"ABCDEFGHIJKLMNOPQRSTUVWXYZ",//TOO EASY
				"1234567890",//too easy
				"1234567890",//too easy
				"1234567890",//too easy
				"1234567890",//too easy
				"check4Me"//close
		);
		for (String pwd : badPwds) {
			assertThat(pwd + " should be insufficient", PasswordQuality.determineQuality(pwd), is(PasswordQuality.insufficient));
		}
	}

	@Test
	public void testDetermineQualityMinimal() throws Exception {
		List<String> minimalPwds = Arrays.asList(
				"1!aAbc",//6 chars and each character class at least once
				"ch$ck4Me"
		);
		for (String pwd : minimalPwds) {
			assertThat(pwd + " should be minimal", PasswordQuality.determineQuality(pwd), is(PasswordQuality.minimal));
		}
	}

	@Test
	public void testDetermineQualityMedium() throws Exception {
		List<String> mediumPwds = Arrays.asList(
				"1!aAbcd12",//8 chars and each character class at least once
				"ch$ck4MeeErRf"
		);
		for (String pwd : mediumPwds) {
			assertThat(pwd + " should be medium", PasswordQuality.determineQuality(pwd), is(PasswordQuality.medium));
		}
	}

	@Test
	public void testDetermineQualityGood() throws Exception {
		List<String> goodPwds = Arrays.asList(
				"1!aAbcd1HU2A(",//12 chars and each character class at least once
				"ch$ck4M3D§aeErRf"
		);
		for (String pwd : goodPwds) {
			assertThat(pwd + " should be good", PasswordQuality.determineQuality(pwd), is(PasswordQuality.good));
		}
	}

	@Test
	public void testDetermineQualityPerfect() throws Exception {
		List<String> perfectPwds = Arrays.asList(
				"An&N0wW3Pre$entAPerfectP4ssw0rd!",//32 chars and each character class at least once
				"JustW§1teAS3n/enc3An?YouA#eFine"
		);
		for (String pwd : perfectPwds) {
			assertThat(pwd + " should be perfect", PasswordQuality.determineQuality(pwd), is(PasswordQuality.perfect));
		}
	}

	@Test
	public void testCalcNumberScore() {
		assertThat(PasswordQuality.calcNumberScore("abcdefghijklmnopqrstuvwxyzABCDEFGHIJKLMNOPQRSTUVWXYZ!§$%&/()=?"), is(-10));
		List<String> numbers = Arrays.asList("1", "2", "3", "4", "5", "6", "7", "8", "9", "0");
		for (String aChar : numbers) {
			assertThat(aChar + "should be a number", PasswordQuality.calcNumberScore(aChar), is(2));
		}
		assertThat(PasswordQuality.calcNumberScore("123"), is(5));
		assertThat(PasswordQuality.calcNumberScore("1234"), is(5));
	}

	@Test
	public void testCalcUppercaseCharScore() {
		assertThat(PasswordQuality.calcUppercaseCharScore("abcdefghijklmnopqrstuvwxyz1234567890!§$%&/()=?"), is(-10));
		List<String> uppercaseChars = Arrays.asList("A", "B", "C", "D", "E", "F", "G", "H", "I", "J", "K", "L", "M", "N", "O", "P", "Q", "R", "S", "T", "U", "V", "W", "X", "Y", "Z");
		for (String aChar : uppercaseChars) {
			assertThat(aChar + "should be an uppercase char", PasswordQuality.calcUppercaseCharScore(aChar), is(2));
		}
		assertThat(PasswordQuality.calcUppercaseCharScore("ABC"), is(5));
		assertThat(PasswordQuality.calcUppercaseCharScore("ABCDE"), is(5));
	}

	@Test
	public void testCalcLowercaseCharScore() {
		assertThat(PasswordQuality.calcLowercaseCharScore("ABCDEFGHIJKLMNOPQRSTUVWXYZ1234567890!§$%&/()=?"), is(-10));
		List<String> uppercaseChars = Arrays.asList("a", "b", "c", "d", "e", "f", "g", "h", "i", "j", "k", "l", "m", "n", "o", "p", "q", "r", "s", "t", "u", "v", "w", "x", "y", "z");
		for (String aChar : uppercaseChars) {
			assertThat(aChar + "should be an lowercase char", PasswordQuality.calcLowercaseCharScore(aChar), is(2));
		}
		assertThat(PasswordQuality.calcLowercaseCharScore("abc"), is(5));
		assertThat(PasswordQuality.calcLowercaseCharScore("abcde"), is(5));
	}

	@Test
	public void testCalcSpecialCharScore() {
		assertThat(PasswordQuality.calcSpecialCharScore("abcdefghijklmnopqrstuvwxyzABCDEFGHIJKLMNOPQRSTUVWXYZ1234567890"), is(-10));
		List<String> exampleSpecialChars = Arrays.asList("^", "!", "!", "\"", "§", "$", "%", "&", "/", "(", ")", "=", "?", "+", "*", "#", "'", "-", "_", ".", ":", ",", ";", "<", ">", "°", "´", "`", "Ö", "Ä", "Ü", "ö", "ä", "ü");
		for (String aChar : exampleSpecialChars) {
			assertThat(aChar + "should be a special char", PasswordQuality.calcSpecialCharScore(aChar), is(2));
		}
		assertThat(PasswordQuality.calcSpecialCharScore("!\"§ $"), is(10));
		assertThat(PasswordQuality.calcSpecialCharScore("!\"§$%ä"), is(10));
	}

	@Test
	public void testCalcLengthScore() {
		assertThat(PasswordQuality.calcLengthScore("123456789012345678901234567890"), is(30));
		assertThat(PasswordQuality.calcLengthScore("1234567890123456789012345"), is(20));
		assertThat(PasswordQuality.calcLengthScore("12345678901234567890"), is(16));
		assertThat(PasswordQuality.calcLengthScore("123456789012345"), is(11));
		assertThat(PasswordQuality.calcLengthScore("1234567890"), is(7));
		assertThat(PasswordQuality.calcLengthScore("123456789"), is(6));
		assertThat(PasswordQuality.calcLengthScore("12345678"), is(5));
		assertThat(PasswordQuality.calcLengthScore("1234567"), is(4));
		assertThat(PasswordQuality.calcLengthScore("123456"), is(0));
		assertThat(PasswordQuality.calcLengthScore("12345"), is(-100));
	}
}
